/* NavigationManager.java
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package de.dhbwloe.campusapp;

import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentTransaction;
import android.view.Gravity;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.PopupWindow;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import de.dhbwloe.campusapp.fragments.PopupFragment;

/**
 * Created by pk910 on 19.01.2016.
 */
public class NavigationManager {
    private class NavPage {
        String name;
        Class<CampusAppFragment> fragmentClass;
        int fragmentType; // 0 = normal; 1 = fullscreen; 2 = popup
    };

    private CampusAppContext AppContext;

    // Derzeitig angezeigtes Fragment
    private NavPage oCurrentPage;
    private Fragment oCurrentFragment;

    // Hintergrund Fragment (bei Popups)
    private NavPage oParentPage;
    private Fragment oParentFragment;

    // Container ID
    private int iFragmentContainerId;

    // Alle registrierte Fragmente mit Name
    private ArrayList<NavPage> lNavigationPages = new ArrayList<NavPage>();

    public NavigationManager(CampusAppContext context, int fragmentContainer) {
        AppContext = context;
        iFragmentContainerId = fragmentContainer;
        oCurrentPage = null;
    }

    public void registerPage(String name, Class<?> fragment, int fragmentType) {
        NavPage page = new NavPage();
        page.name = name;
        page.fragmentClass = (Class<CampusAppFragment>) fragment;
        page.fragmentType = fragmentType;
        lNavigationPages.add(page);
    }

    public String getCurrentPageName() {
        if(oCurrentPage == null)
            return null;
        if(oCurrentPage.fragmentType == 3 && oParentFragment != null){
            if(oCurrentFragment.isAdded())
                return oCurrentPage.name;
            else {
                closeDialog();
            }
        }
        return oCurrentPage.name;
    }

    public CampusAppFragment getCurrentFragment() {
        if(oCurrentPage == null)
            return null;
        if(oCurrentPage.fragmentType != 3)
            return (CampusAppFragment)oCurrentFragment;
        else
            return ((PopupFragment)oCurrentFragment).getCurrentFragment();
    }

    public void navigatePage(String name) {
        navigatePage(name, null, true);
    }

    public void navigatePage(String name, Bundle args) {
        navigatePage(name, args, true);
    }

    public void navigatePage(String name, Bundle args, boolean history) {
        // Suche Fragment
        NavPage page = getPageByName(name);
        if(page == null)
            return;
        Fragment fragment;

        // Wenn das Fragment als Popup angeziegt werden soll, muss zunächst das Popup Fragment geladen werden.
        if(page.fragmentType == 3) {
            PopupFragment popupFragment = new PopupFragment();
            if(args == null)
                args = new Bundle();
            args.putString("target", "#"+page.name);
            fragment = popupFragment;
        } else {
            fragment = getFragmentOfPage(page);
        }

        fragment.setArguments(args);

        FragmentActivity fragmentActivity = (FragmentActivity) AppContext.getMainActivity();
        FragmentTransaction transaction = fragmentActivity.getSupportFragmentManager().beginTransaction();

        if (oCurrentPage != null && oCurrentPage.fragmentType == 3) {
            transaction.remove(oCurrentFragment);
            oCurrentPage = oParentPage;
            oCurrentFragment = oParentFragment;
        }
        if (page.fragmentType == 3) {
            transaction.add(fragment, "popup");
            oParentPage = oCurrentPage;
            oParentFragment = oCurrentFragment;
            history = false;
        } else {
            // remove fragments
            List<Fragment> al = fragmentActivity.getSupportFragmentManager().getFragments();
            if(al != null) {
                for (Fragment frag : al) {
                    if (frag != null && frag.isAdded()) {
                        transaction.remove(frag);
                    }
                }
            }
            transaction.add(iFragmentContainerId, fragment);
        }
        if (history) // Hinzufügen zur App History (Zurück Button)
            transaction.addToBackStack(null);

        oCurrentPage = page;
        oCurrentFragment = fragment;

        transaction.commit();
    }

    private NavPage getPageByName(String name) {
        NavPage page = null;
        for(int i = 0; i < lNavigationPages.size(); i++) {
            if(lNavigationPages.get(i).name.equalsIgnoreCase(name)) {
                page = lNavigationPages.get(i);
                break;
            }
        }
        if(page == null)
            return null;

        return page;
    }

    private CampusAppFragment getFragmentOfPage(NavPage page) {
        Class<CampusAppFragment> fragmentClass = page.fragmentClass;
        Constructor fragmentConstructor;
        CampusAppFragment fragment;
        try {
            fragmentConstructor = fragmentClass.asSubclass(fragmentClass).getConstructor();
            fragment = (CampusAppFragment)fragmentConstructor.newInstance(new Object[]{});
        } catch (Exception e) {
            return null;
        }
        return fragment;
    }

    public CampusAppFragment getPageFragment(String name) {
        NavPage page = getPageByName(name);
        if(page == null)
            return null;
        return getFragmentOfPage(page);
    }

    public PopupFragment getDialog() {
        if(oCurrentPage != null && oCurrentPage.fragmentType == 3) {
            PopupFragment fragment = (PopupFragment) oCurrentFragment;
            return fragment;
        }
        return null;
    }

    public boolean closeDialog() {
        // Popup schließen
        if(oCurrentPage != null && oCurrentPage.fragmentType == 3) {
            PopupFragment fragment = (PopupFragment) oCurrentFragment;
            boolean wasAdded = false;
            if(oCurrentFragment.isAdded()) {
                fragment.destroyView();

                FragmentActivity fragmentActivity = (FragmentActivity) AppContext.getMainActivity();
                FragmentTransaction transaction = fragmentActivity.getSupportFragmentManager().beginTransaction();

                transaction.remove(oCurrentFragment);
                transaction.commit();
                wasAdded = true;
            }

            oCurrentPage = oParentPage;
            oCurrentFragment = oParentFragment;

            return wasAdded;
        }
        return false;
    }

    public boolean back() {
        return closeDialog();
    }

}
